package com.flyer.foster.service.impl;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.bean.BeanUtil;
import com.flyer.exception.BusinessException;
import com.flyer.foster.dto.RoleAddDTO;
import com.flyer.foster.dto.RoleMenuAddDTO;
import com.flyer.foster.dto.RoleRespDTO;
import com.flyer.foster.entity.Role;
import com.flyer.foster.enums.TFEnum;
import com.flyer.foster.mapper.IRoleMapper;
import com.flyer.foster.service.IRoleMenuService;
import com.flyer.foster.service.IRoleService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import java.util.ArrayList;
import java.util.List;

/**
 * <p>
 * 角色表 服务实现类
 * </p>
 *
 * @author flyer
 * @since 2024-05-06
 */
@Service
public class RoleServiceImpl extends ServiceImpl<IRoleMapper, Role> implements IRoleService {
    @Autowired
    private IRoleMenuService iRoleMenuService;

    @Override
    public List<RoleRespDTO> getRoleList() {
        int tenantId = StpUtil.getSession().getInt("tenantId");
        List<RoleRespDTO> list = new ArrayList<>();
        List<Role> roleList = this.lambdaQuery()
                .eq(Role::getTenantId, tenantId)
                .eq(Role::getIsAdmin, TFEnum.FALSE.getId())
                .list();
        RoleRespDTO roleRespDTO;
        for (Role role : roleList) {
            roleRespDTO = new RoleRespDTO();
            BeanUtil.copyProperties(role, roleRespDTO);
            list.add(roleRespDTO);
        }
        return list;
    }

    @Transactional
    @Override
    public boolean addRole(RoleAddDTO addDTO) {
        int tenantId = StpUtil.getSession().getInt("tenantId");
        addDTO.setRoleName(addDTO.getRoleName().trim());
        addDTO.setRoleCode(addDTO.getRoleCode().trim());
        if (addDTO.getMenuIdList().size() == 0) {
            throw new BusinessException("请选择菜单");
        }
        // 1.判断（角色名称+角色编码）是否存在
        Role role = this.lambdaQuery()
                .eq(Role::getRoleName, addDTO.getRoleName())
                .eq(Role::getRoleCode, addDTO.getRoleCode())
                .one();
        if (role != null) {
            throw new BusinessException("角色编码+角色名称不能重复");
        }
        role = new Role();
        BeanUtil.copyProperties(addDTO, role);
        role.setTenantId(tenantId);
        // 新增角色
        this.save(role);

        RoleMenuAddDTO roleMenuAddDTO = new RoleMenuAddDTO();
        roleMenuAddDTO.setRoleId(role.getId());
        roleMenuAddDTO.setMenuIdList(addDTO.getMenuIdList());
        // 保存角色菜单
        iRoleMenuService.addRoleMenu(roleMenuAddDTO);
        return true;
    }
}
